@@ -8,7 +8,9 @@ function(sgp_object,
 	sgp.percentiles=TRUE, 
 	sgp.projections=TRUE,
 	sgp.projections.lagged=TRUE,
+	sgp.percentiles.baseline=FALSE,
 	simulate.sgps=TRUE,
+	parallel.config,
 	save.intermediate.results=FALSE,
 	sgp.summaries=list(MEDIAN_SGP="median_na(SGP)",
 		MEDIAN_SGP_TARGET="median_na(SGP_TARGET)",
@@ -86,7 +88,9 @@ function(sgp_object,
 			sgp.percentiles=sgp.percentiles,
 			sgp.projections=sgp.projections,
 			sgp.projections.lagged=sgp.projections.lagged,
-			simulate.sgps=simulate.sgps)
+			sgp.percentiles.baseline=sgp.percentiles.baseline,
+			simulate.sgps=simulate.sgps,
+			parallel.config = parallel.config)
 
                 if (save.intermediate.results) save(sgp_object, file="sgp_object.Rdata")
 	}
