@@ -8,7 +8,7 @@
 	started.at <- proc.time()
 	message(paste("Started prepareSGP", date()))
 
-	if (class(data)=="SGP") {
+	if (class(data)[1]=="SGP") {
 
 		message(paste("Finished prepareSGP", date(), "in", timetaken(started.at), "\n"))
 
@@ -33,8 +33,7 @@
 	##  Check variable names
 
 	if (!missing(var.names)) {
-		var.names <- unlist(var.names)
-		var.names <- data.frame(nms.orig=toupper(as.vector(var.names)), nms.sgp=toupper(as.vector(names(var.names))))
+		var.names <- data.frame(nms.orig=toupper(unlist(var.names)), nms.sgp=toupper(names(var.names)))
 		
 		## Include default variable names (as needed)
 
@@ -47,7 +46,7 @@
 	
 	## Compile the original variable names and the corresponding (capitalized) variable names used in SGP
 
-	var.names.original <- data.frame(column=1:length(nms.original), nms.original=nms.original, nms.orig=toupper(nms.original))
+	var.names.original <- data.frame(column=seq_along(nms.original), nms.original=nms.original, nms.orig=toupper(nms.original))
 	var.names$flag <- 1
 	tmp.var.names <- merge(var.names.original, var.names, all.x=TRUE)
 	tmp.var.names$nms.sgp[is.na(tmp.var.names$flag)] <- NA
@@ -61,7 +60,7 @@
 	## Check to see if any of the required variables are missing
 
 	if (!all(req.nms %in% tmp.var.names$nms.sgp)) {
-		stop(paste("The {data} object is missing the following column name: ",req.nms[(req.nms %in% tmp.var.names$nms.sgp)==FALSE],
+		stop(paste("The {data} object is missing the following column name: ", req.nms[(req.nms %in% tmp.var.names$nms.sgp)==FALSE],
 		". You may need to identify the variable using the {var.names} argument.", sep=""))
 	}
 
@@ -83,7 +82,7 @@
 	
 	##  Create the SGP object
 
-	sgp_object <- new("SGP", Data=data, Variable_Name_Lookup=tmp.var.names)
+	sgp_object <- new("SGP", Data=data, Names=tmp.var.names)
 
 	
 	##  Print finish time
